# $Id: code_change.rb 34 2008-04-03 06:01:19Z zev $
#
# This file implements the logic to check for additions and removals
# of methods and classes in a source base.

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Zev Blut (zb@ubit.com)

class CodeChangeAction < BuildAction

  def initialize
    @code_changes = nil
  end

  def build_action(context)
    @code_changes = find_code_changes(context.ri_dir_1, context.ri_dir_2)
  end

  def summary_display(context)
    template = TemplateFile.new(self.class.summary_template_file)
    context.amrita_data[:code_change_results] = @code_changes
    summary_expand(template, context)
  end

  def self.detailed_template_file
    nil
  end

  protected

  def filter_method_from_classes?(method, classes)
    m = /(.*)\/.+\.yaml$/.match(method)
    base = m[1]
    classes.include?(base)
  end

  def strip_ri_method_name(meth)
    m = /(.*)-(i|c)\.yaml/.match(meth)
    m[1]
  end

  def find_code_changes(ri_dir="/tmp/ri", new_ri_dir="/tmp/new_ri")
    results = Hash.new

    new_ri_files = find_ri_files(new_ri_dir)
    old_ri_files = find_ri_files(ri_dir)

    new_classes = new_ri_files.reject {|f| f =~ /\.(yaml|rid)$/ || f == new_ri_dir }
    old_classes = old_ri_files.reject {|f| f =~ /\.(yaml|rid)$/ || f == ri_dir }

    new_methods = new_ri_files.find_all {|f| f =~/\.yaml$/}
    old_methods = old_ri_files.find_all {|f| f =~/\.yaml$/}

    rep_new_classes = new_classes - old_classes
    rep_remove_classes = old_classes - new_classes

    rep_new_methods = new_methods - old_methods
    rep_remove_methods = old_methods - new_methods

    rep_new_methods = rep_new_methods.find_all {|f| !filter_method_from_classes?(f, rep_new_classes)}
    rep_remove_methods = rep_remove_methods.find_all {|f| !filter_method_from_classes?(f, rep_remove_classes)}

    rep_new_class_methods, rep_new_instance_methods = rep_new_methods.partition {|m| m =~ /-c\.yaml$/}
    rep_remove_class_methods, rep_remove_instance_methods = rep_remove_methods.partition {|m| m =~ /-c\.yaml$/}

    rep_new_instance_methods = rep_new_instance_methods.map {|m| strip_ri_method_name(m)}
    rep_remove_instance_methods = rep_remove_instance_methods.map {|m| strip_ri_method_name(m)}

    rep_new_class_methods = rep_new_class_methods.map {|m| strip_ri_method_name(m)}
    rep_remove_class_methods = rep_remove_class_methods.map {|m| strip_ri_method_name(m)}

    results[:new_classes] = make_table_iteration( rep_new_classes )
    results[:removed_classes] = make_table_iteration( rep_remove_classes )
    results[:new_instance_methods] = make_table_iteration( rep_new_instance_methods )
    results[:removed_instance_methods] = make_table_iteration( rep_remove_instance_methods )
    results[:new_class_methods] = make_table_iteration( rep_new_class_methods )
    results[:removed_class_methods] = make_table_iteration( rep_remove_class_methods )

    results
  end

  def make_table_iteration(data)
    {:item => data.map { |c| { :name => c } } }
  end

  def find_ri_files(ri_dir)
    files = Array.new
    Find.find(ri_dir) do |file|
      files << ProjectBuilderUtils.strip_base_dir(ri_dir, file)
    end
    files
  end

end

