# $Id: cycle_detection.rb 32 2008-04-02 07:50:18Z zev $
#
# Implements a Ruby require cycle detection and graphing action.

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Zev Blut (zb@ubit.com)

require 'stringio'

require "#{LIB_DIR}/cycle_detector"

class CycleDetectionAction < BuildAction

  def initialize
    @cycle_detector = CycleDetector.new
    @cycle_detector.print_strategy = self
    @output_directory = nil
    @cycles = nil
  end

  def build_action(context)
    # convert obj?
    @cycle_detector.make_graph(context.project_directory)
  end


  def summary_display(context)
    template = TemplateFile.new(self.class.summary_template_file)
    @cycle_detector.print_unique_cycles( @cycle_detector.find_cycles )

    if @cycles.empty?
      context.amrita_data[:cycle_results] = "There are no require cycles."
    else
      data = @cycles.map do |cycle|
        {
          :name => "#{ cycle.first} <=> #{ cycle.last }",
          :list => cycle
        }
      end
      context.amrita_data[:cycle_results] = { :entry => data  }
    end


    loc_base = "#{context.project_name}_cycle.html"
    context.amrita_data[:cycle_details] =
      Amrita.e(:a, :href => loc_base ) { "Require Graph Details" }

    summary_expand(template, context)
  end

  def detailed_display(context)
    template = TemplateFile.new(self.class.detailed_template_file)

    @output_directory = context.output_directory
    @cycle_detector.print_graph

    ["demos.jar", "prefuse.jar"].each do |jarfile|
      FileUtils.cp(EXTERNAL_DIR + jarfile, context.output_directory)
    end

    det_res = ProjectBuilderUtils.expand_template(template, context.amrita_data)
    det_file = "#{context.output_directory}/#{context.project_name}_cycle.html"
    [det_file, det_res]
  end


  def print_graph(graph)
    graph_ml = CycleDetector::GraphMLPrintStrategy.new
    begin
      oldstd = $stdout
      $stdout = StringIO.new
      graph_ml.print_graph(graph)
      gxml = $stdout.string
    ensure
      $stdout = oldstd
    end
    # now dump that into the out dir
    File.open(@output_directory + "/require_graph.xml", "w") { |f| f.puts gxml }
  end

  def print_cycles(cycles)
    @cycles = cycles
  end

end
