# $Id: flagged_comments.rb 32 2008-04-02 07:50:18Z zev $
#
# This file implements the logic to scan for special comments in
# a source base.

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Zev Blut (zb@ubit.com)


class FlaggedCommentsAction < BuildAction
  COMMENTFLAGS = [ "BUG", "FIXME", "???", "TODO", "XXX" ]
  COMMENTREGEXP = Regexp.new("\#\s*(" +
                               COMMENTFLAGS.map { |i| Regexp.escape(i)}.join("|") +
                               ")\s*:?(.*)")

  def initialize
    @comment_count = Hash.new { |h,k| h[k] = Array.new }
  end

  def build_action(context)
    projbase = context.project_directory.split("/").last
    (context.ruby_files + context.test_files).each do |file|
      fcount = collect_comments_from_file(file)
      fcount.each do |cflag, val|
        fnamemap = val.map do |idx, txt|
          nfile = file.split(context.project_directory).last
          ["#{ projbase }/#{ nfile }\##{ idx }" , txt]
        end
        @comment_count[cflag].concat( fnamemap )
      end
    end
  end

  def summary_display(context)
    template = TemplateFile.new(self.class.summary_template_file)

    ares = COMMENTFLAGS.map do |ctype|
      { :type => ctype,
        :count => @comment_count[ctype].size
      }
    end

    context.amrita_data[:com_results] = { :entry => ares }
    loc_base = "#{context.project_name}_com.html"
    context.amrita_data[:com_details] =
     (Amrita::e(:a, :href => loc_base) { "Flagged Comments Details" } )

    summary_expand(template, context)
  end

  def detailed_display(context)
    template = TemplateFile.new(self.class.detailed_template_file)


    ares = COMMENTFLAGS.map do |ctype|
      { :type => ctype,
        :count => @comment_count[ctype].size,
        :entries => @comment_count[ctype].map do |file, txt|
          {
            :file => Amrita::e(:a, :href => link_to_file(file)) { file },
            :txt => txt
          }
        end
      }
    end

    context.amrita_data[:com_details] = { :entry => ares }
    det_res = ProjectBuilderUtils.expand_template(template, context.amrita_data)
    det_file = "#{context.output_directory}/#{context.project_name}_com.html"
    [det_file, det_res]
  end


  def collect_comments_from_file(file)
    res = Hash.new { |h,k| h[k] = Array.new }
    IO.readlines(file).each_with_index do |line, idx|
      if m = COMMENTREGEXP.match(line)
        res[m[1]]<< [idx + 1 , m[2]]
      end
    end
    res
  end

  #annotate does the line numbering and pointing
  def link_to_file(file)
    file, line = file.split("#")
    view_url = annotate_url +
      file.to_s + "?annotate=HEAD\#" + line.to_s
  end

  def annotate_url
    "http://localhost/cgi-bin/viewcvs/cgi/viewcvs.cgi/"
  end

end
