require "#{File.dirname(__FILE__)}/unit_test"

# requires multiruby, part of ZenTest: http://www.zenspider.com/ZSS/Products/ZenTest/
class MultiRubyUnitTestBuildAction < UnitTestBuildAction

  MULTIRUBY = '/usr/local/bin/multiruby'
  SITE_RUBY = '/usr/local/lib/ruby/site_ruby/1.8'

  def self.summary_template_file
    superclass.summary_template_file
  end

  def self.detailed_template_file
    superclass.detailed_template_file
  end

  #################
  # Protected

  def ruby_version
    return @ruby_version if defined? @ruby_version
    version = `#{MULTIRUBY} -v`
    @ruby_version = version.scan(/VERSION = ([\.0-9]*)/).join(', ')
  end

  def ruby_command_line(pre_requires, file)
    req_cmd = "-I#{SITE_RUBY} -I#{SITE_RUBY}/i686-linux/"
    if pre_requires
      req_cmd << " -r #{pre_requires}"
    end
    "#{MULTIRUBY} #{req_cmd} #{file}"
  end

  def parse_test_results(data, file)
    version_reg = /([0-9\.]*)#{RES_REGEXP}/im
    version_array = data.split('VERSION = ').map{|s| s.match(version_reg)}.compact
    if version_array.empty?
      [no_match_result(data, file)]
    else
      version_array.map do |m|
        {
          :file_name => file,
          :version_name => m[1],
          :time => m[3],
          :tests => m[5],
          :assertions => m[6],
          :failures => template_error_num(m[7]),
          :errors => template_error_num(m[8]),
          :test_results => parse_out_test_info(m[4]).map {|i| {:test_info => i}}
        }
      end
    end
  end

  alias :old_time_out_readlines :time_out_readlines

  def time_out_readlines(io)
    # We give it a minute and a half now
    old_time_out_readlines(io, 90)
  end

end
