# $Id: outside_links.rb 34 2008-04-03 06:01:19Z zev $
#
# This file implements an action that creates links to reports
# not generated by the project builder

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Zev Blut (zb@ubit.com)

class OutsideLinksAction < BuildAction
  BASE_URL = ""

  def initialize
    no_http = Proc.new do |link, project_name|
      "#{BASE_URL}/#{link}/#{project_name}/"
    end
    http = Proc.new do |link, project_name|
      "#{link}/#{project_name}/"
    end

    @links = [
      ["rdocs", "Documentation", no_http],
      ["statcvs_output", "StatCVS", no_http],
      ["http://localhost/viewvc", "ViewVC", http],
      ["http://localhost/pragprog/data/rss", "CVS Commit RSS",
        Proc.new { |link, project_name|  "#{link}/#{project_name}.rss" }
      ]
    ]
  end

  def build_action(context)
    #Do nothing
  end

  def summary_display(context)
    template = TemplateFile.new(self.class.summary_template_file)
    context.amrita_data[:outside_links] = @links.map do |link, lname, action|
      {
        :item => Amrita::e(:h3) do
          url = action.call(link, context.project_name)
          Amrita::e(:a, :href => url) { lname }
        end
      }
    end

    summary_expand(template, context)
  end

  def self.detailed_template_file
    nil
  end

end
