# $Id: strange_requires.rb 32 2008-04-02 07:50:18Z zev $
#
# This file implements the logic to check for additions and removals
# of methods and classes in a source base.

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Michael Reinsch

require "#{LIB_DIR}/strange_requires_detector"

class StrangeRequires < BuildAction

  class AmritaPrintStrategy
    def initialize
      @subdir_reqs = []
      @parentdir_reqs = []
    end

    def subdir_requirement(from, to)
      @subdir_reqs << { :from => from, :to => to }
    end
    def parent_dir_requirement(from, to)
      if not /\/lib\// =~ to
        @parentdir_reqs << { :from => from, :to => to }
      end
    end

    def require_data_summary
      { :sub_dir_requires => @subdir_reqs.size,
        :parent_dir_requires => @parentdir_reqs.size }
    end

    def require_data_detail
      [ { :require_type => "Subdirectory Requirements",
          :entry => @subdir_reqs },
        { :require_type => "Parent Directory Requirements",
          :entry => @parentdir_reqs} ]
    end
  end

  def initialize
    @amrita_ps = AmritaPrintStrategy.new
    @strange_requires_detector = StrangeRequiresDetector.new(@amrita_ps)
  end

  def build_action(context)
    @strange_requires_detector.check(context.project_directory, context.project_directory)
  end

  def details_file(context)
    "#{context.output_directory}/#{details_link(context)}"
  end
  def details_link(context)
    "#{context.project_name}_requires.html"
  end

  def summary_display(context)
    template = TemplateFile.new(self.class.summary_template_file)
    context.amrita_data[:require_results] = @amrita_ps.require_data_summary
    context.amrita_data[:require_details] =
      Amrita::e(:a, :href => details_link(context)) { "Requires Details" }
    summary_expand(template, context)
  end

  def detailed_display(context)
    template = TemplateFile.new(self.class.detailed_template_file)
    context.amrita_data[:require_details] = @amrita_ps.require_data_detail
    det_res = ProjectBuilderUtils.expand_template(template, context.amrita_data)
    det_file = details_file(context)
    [det_file, det_res]
  end

end
