# $Id: syntax_check.rb 39 2008-04-30 10:55:14Z zev $
#
# This file implements the logic to perform a syntax check.

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Zev Blut (zb@ubit.com)

class SyntaxCheckAction < BuildAction

  def initialize
    @syntax_results = nil
  end

  def build_action(context)
    @syntax_results = check_ruby_syntax(context.ruby_files)
  end

  def summary_display(context)
    template = TemplateFile.new(self.class.summary_template_file)

    if @syntax_results.empty?
      context.amrita_data[:syntax_details] = "Syntax Check"
      context.amrita_data[:syntax_results] = "There are no syntax errors."
    else
      context.amrita_data[:syntax_results] = {
        :entry => @syntax_results
      }

      syntax_base = "#{context.project_name}_syntax.html"
      if context.amrita_data[:syntax_results][:entry]
        context.amrita_data[:syntax_results][:entry].each do |entry|
          entry[:file_name_link] = Amrita::a(:href => "#{syntax_base}\##{entry[:file_name]}")  { entry[:file_name] }
        end
      end

      context.amrita_data[:syntax_details] =
        (Amrita::e(:a, :href=>syntax_base) { "Syntax Check Details" })
    end
    summary_expand(template, context)
  end

  def detailed_display(context)
    context.amrita_data[:syntax_results] = {
      :entry => @syntax_results
    }
    det_res = expand_syntax_check_template(context.amrita_data[:syntax_results])
    det_file = "#{context.output_directory}/#{context.project_name}_syntax.html"
    [det_file, det_res]
  end

  def score
    return 10 if @syntax_results.empty?
    errors = @syntax_results.map { |e| e[:error_count] }.inject(0) { |s,v| s + v}
    if errors > 0
      #errors > 10 ? -10 : 10 - errors
      # For now any sytnax error is bad.
      -100
    else
      # must have warnings 3 is an arbitrary choice at the moment.
      3
    end
  end

  def check_ruby_syntax(ruby_files)
    # need kcode addition
    results = Array.new
    ruby_files.each do |f|
      entry = Hash.new
      #STDERR.puts "Checking with command:\nruby -KS -c #{f}"
      Open3.popen3("/usr/bin/env ruby -c #{f}") do |stdin, stdout, stderr|
        stdout_data = time_out_readlines(stdout)
        stderr_data = time_out_readlines(stderr)
        if stdout_data.first !~ /Syntax OK/ || stderr_data.size > 0
          warnings, errors = stderr_data.partition {|r| r =~ /: warning:/}
          entry[:file_name] = f
          entry[:warnings] = warnings
          entry[:warning_count] = warnings.size
          entry[:errors] = errors
          entry[:error_count] = errors.size
          results << entry
        end
      end
    end
    results
  end

  def expand_syntax_check_template(data)
    template = TemplateFile.new(self.class.detailed_template_file)
    data[:entry].each do |entry|
      entry[:file_name_ref] = Amrita::a(:name => entry[:file_name]) { entry[:file_name] }
      entry[:errors] = entry[:errors].to_a.map {|d| {:error => d} }
      entry[:warnings] = entry[:warnings].to_a.map {|d| {:warning => d} }
    end
    ProjectBuilderUtils.expand_template(template, data)
  end

end
