class BuildAction
  extend InheritanceTracker

  def build_action(context)
    raise "Must be implemented by subclass."
  end

  def summary_display(context)
    raise "Must be implemented by subclass."
  end

  def detailed_display(context)
    [nil, nil]
  end

  # a number from 1 - 10 that represents some type of rating that will help
  # determine if the koala's expression.
  # For now if nil, then not scorable.  This might be better as an additional method
  # called scoreable?
  # Default is unscoreable
  def score
    nil
  end

  def self.command_line_action_name
    self.name.gsub(/BuildAction|Action/, '').kwala_underscore
  end

  def self.summary_template_file
    "#{TEMPLATE_DIR}/#{command_line_action_name}_summary.html"
  end

  def self.detailed_template_file
    "#{TEMPLATE_DIR}/#{command_line_action_name}_detailed.html"
  end

  def BuildAction.create_action_from_command_line_name(name)
    imp = BuildAction.get_implementors.find do |imp|
      imp.command_line_action_name == name
    end

    if imp
      imp.new
    else
      nil
    end
  end

  def BuildAction.command_line_action_names
    BuildAction.get_implementors.map do |imp|
      imp.command_line_action_name
    end.sort
  end


  protected

  def time_out_readlines(io, timeout_period = 300)
    res = nil
    begin
      timeout(timeout_period) do
        res = io.readlines
      end
    rescue TimeoutError => err
      res = ["IO readlines timed out."]
    end
    res
  end

  def summary_expand(template, context)
    res = ProjectBuilderUtils.expand_template(template, context.amrita_data)
    Amrita::SanitizedString.new(res)
  end

  # These are bad param names, need to make them better
  # Extremely naive function to put the errors to some scale around 10.
  def score_scaling(errors, bounds)
     10 - ((errors.to_f / bounds.to_f) * 10).ceil
  end

  def notify(result, data = nil)
    Notification.notify(self.class.command_line_action_name, result, data)
  end

end

class KwalaAbortBuildException < StandardError
end
