#!/usr/bin/env ruby -w

# Copyright (c) 2006, Ubiquitous Business Technology (http://ubit.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * Neither the name of Ubit nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# == Author
# Michael Reinsch (mr@ubit.com)

require File.dirname(__FILE__) + "/cycle_detector"   # for ReqWalker

class Dir
  def == (other)
    case other
    when String;  return path == other
    when Dir;     return path == other.path
    when FakeDir; return path == other.path
    end
    false
  end

  def is_subdir_of?(dir)
    return path[0..dir.path.length-1] == dir.path
  end

  def cut(basedir)
    path
  end
end


class StrangeRequiresDetector

  attr_writer :print_strategy

  def initialize(printer)
    @graph = Hash.new
    @printer = printer
    @file_filter = Proc.new do |files|
      files.find_all do |f|
        /tests\// !~ f && /extensions\// !~ f &&
          /tools\// !~ f && /\/ms\// !~ f && /\/site\// !~f
      end
    end
    @req_filter = Proc.new do |reqs|
      reqs.find_all do |r|

      end
    end
  end

  def cut_base(filename, basename)
    filename.gsub(/^#{basename}/, '')
  end

  def check(dir, basedir = "")
    files = ReqWalker.find_ruby_files(dir)
    @file_filter.call(files).each do |f|
      ReqWalker.get_requires(f).each do |r|
        r = ReqWalker.find_file(r)
        if r && /\/lib\/ruby\// !~ r
          f_base = Dir.new(File.dirname(f))
          r_base = Dir.new(File.dirname(r))
          # well, we assume that a directory models our "class category"...
          if f_base != r_base
            if r_base.is_subdir_of?(f_base)
              @printer.subdir_requirement(cut_base(f,basedir), cut_base(r,basedir))
            else
              @printer.parent_dir_requirement(cut_base(f,basedir), cut_base(r,basedir))
            end
          end
        end
      end
    end
  end

end


if __FILE__ == $0
  class STDOUTPrintStrategy
    def subdir_requirement(from, to)
      puts "yellow: #{from} -> #{to}"
    end
    def parent_dir_requirement(from, to)
      puts "red   : #{from} -> #{to}"
    end
  end

  require 'getoptlong'

  srd = StrangeRequiresDetector.new(STDOUTPrintStrategy.new)
  dir = "./"

  begin
    options = GetoptLong.new(
                             ["-d","--dir", GetoptLong::REQUIRED_ARGUMENT]
                             )

    options.each do |opt,arg|
      case opt
      when "-d"
        dir = arg
      else
        STDERR.puts "Unknown command #{ opt }"
        exit 1
      end
    end
  rescue => err
    STDERR.puts "Error invalid command"
    exit 1
  end

  srd.check(dir)

end
